use std::io::Read;
use std::string::String;
use std::collections::HashMap;
use curl::easy::{Easy, List};

pub fn request(auth: HashMap<String, String>) -> u32 {
    let url = auth.get("ha_url").expect("[ERROR] Failed to read the URL").as_str();
    
    // escape character for { is {{ and for } is }}
    let entity_id = auth.get("light_entity_id").expect("[ERROR] Failed to read the entity id").as_str();
    let body = format!("{{\"entity_id\": {entity_id}}}");
    let mut data = body.as_bytes();
    
    let token = auth.get("access_token").expect("[ERROR] Failed to read the token").as_str();
    let auth = format!("Authorization: Bearer {}", &token[1..token.len()-1]);
    
    let mut list = List::new();
    list.append(&auth).unwrap();
    list.append("Content-Type: application/json").unwrap();
    
    let mut request = Easy::new();
    // proper error handling
    
    request.url(&url[1..url.len()-1]).unwrap_or_else(|e| {
        panic!("[ERROR] Failed to set the URL for the CURL request:\n{e}");
    });
    
    request.post(true).unwrap_or_else(|e| {
        panic!("[ERROR] Failed to set the CURL request as POST:\n{e}");
    });
    
    request.http_headers(list).unwrap_or_else(|e| {
        panic!("[ERROR] Failed to apply HTTP headers:\n{e}");
    });
    request.post_field_size(data.len() as u64).unwrap_or_else(|e| {
        panic!("[ERROR] Failed to set the POST field size for CURL request:\n{e}");
    });

    let mut transfer = request.transfer();
    
    transfer.read_function(|buf| {
        Ok(data.read(buf).unwrap_or(0))
    }).unwrap_or_else(|e| {
        panic!("[ERROR] Failed to read the data:\n{e}");
    });
    
    transfer.perform().unwrap_or_else(|e| {
        panic!("[ERROR] Failed to perform the CURL request:\n{e}");
    });
    
    // drop transfer so we can access request.response_code below
    drop(transfer);

    match request.response_code() { 
        Ok(t) => t,
        Err(e) => panic!("[ERROR] Failed to retrieve the response code:\n{e}"),
    }
}